'use strict';

var assert = require('assert');
var Validation = require('./Validation').Validation;

var _require = require('./Validators'),
    AsyncValue = _require.AsyncValue,
    EmailValue = _require.EmailValue;

var asyncTimer = void 0;
var delayValue = 1000;
var delayTime = 250;
var validationSchema = {
    email: {
        required: true,
        type: EmailValue
    },
    name: {
        required: true
    },
    delay: {
        type: AsyncValue,
        asyncFunction: function asyncFunction(data, next) {
            asyncTimer = setTimeout(function () {
                if (data === delayValue) {
                    next(null);
                } else {
                    next({
                        message: 'Validation failed, the data was not the delayValue'
                    });
                }
            }, delayTime);
        },
        cleanup: function cleanup() {
            clearTimeout(asyncTimer);
        }
    }
};

var data = {
    email: 'aaxbzsgher.developer@gmail.com',
    name: '',
    delay: delayValue
};

var validation = new Validation(validationSchema);

describe('Validation', function () {

    it('data should be considered invalid', function (done) {
        var results = [];
        var obs$ = validation.validate(data);
        obs$.subscribe(function (validationResult) {
            results.push(validationResult);
            if (results.length >= 3) {
                assert.equal(results.reduce(function (pre, result) {
                    return pre && result.isValid;
                }, true), false);
                done();
            }
        });
    });

    it('email should be considered valid', function (done) {
        var obs$ = validation.validate({ email: data.email });
        obs$.subscribe(function (validationResult) {
            assert.equal(validationResult.isValid, true);
            done();
        });
    });

    it('name should be considered invalid', function (done) {
        var obs$ = validation.validate({ name: data.name });
        obs$.subscribe(function (validationResult) {
            assert.equal(validationResult.isValid, false, 'name is invalid');
            assert.equal(validationResult.errors.length, 1, 'has error message: ' + validationResult.error);
            done();
        });
    });

    it('delay should be considered valid after a time', function (done) {
        var obs$ = validation.validate({ delay: data.delay });
        var complete = false;
        var subscription = obs$.subscribe(function (validationResult) {
            assert.equal(validationResult.isValid, true);
            complete = true;
            done();
        });

        var tShouldNot = setTimeout(function () {
            if (complete) {
                subscription.unsubscribe();
                done(new Error('The delayed code should not have run yet.'));
            }
            clearTimeout(tShouldNot);
        }, delayTime - 10);

        var tShouldHave = setTimeout(function () {
            if (!complete) {
                subscription.unsubscribe();
                done(new Error('The delayed code did not run.'));
            }
            clearTimeout(tShouldHave);
        }, delayTime);
    });
});